from typing import Dict, List

import networkx as nx
from rb.cna.cna_graph import CnaGraph
from rb.complexity.complexity_index import compute_indices
from rb.core.block import Block
from rb.core.cscl.cna_indices_enum import CNAIndices
from rb.core.cscl.contribution import Contribution
from rb.core.cscl.conversation import Conversation
from rb.core.lang import Lang
from rb.core.pos import POS
from rb.core.text_element import TextElement
from rb.utils.rblogger import Logger


def get_block_importance(graph: nx.DiGraph, a: Block, b: Block) -> float:
	return graph.edges[a, b]["weight"] if (a, b) in graph.edges else 0.


def evaluate_interaction(conversation: Conversation):
	cna_graph = conversation.container.graph if conversation.container is not None else conversation.graph
	
	participants = conversation.get_participants()
	contributions = conversation.get_contributions()
	conversation.init_scores()

	if (len(participants) == 0):
		return

	for i, contribution1 in enumerate(contributions):
		p1 = contribution1.get_participant().get_id()

		for contribution2 in contributions[:i]:
			weight = get_block_importance(cna_graph.filtered_graph, contribution1, contribution2)
			if weight > 0:
				p2 = contribution2.get_participant().get_id()
				conversation.update_score(p1, p2, weight)


def evaluate_involvement(conversation: Conversation):
	cna_graph = conversation.container.graph if conversation.container is not None else conversation.graph
	participants = conversation.get_participants()

	if (len(participants) == 0):
		return

	for contribution in conversation.get_contributions():
		p = contribution.get_participant()

		current_value = p.get_index(CNAIndices.SCORE)
		p.set_index(CNAIndices.SCORE, current_value + cna_graph.importance[contribution])

		current_value = p.get_index(CNAIndices.SOCIAL_KB)
		parent_contribution = contribution.get_parent()

		if parent_contribution != None:
			current_value += get_block_importance(cna_graph.filtered_graph, contribution, parent_contribution)
			p.set_index(CNAIndices.SOCIAL_KB, current_value)

def evaluate_textual_complexity(conversation: Conversation):
	participants = conversation.get_participants()
	
	for p in participants:
		contributions = conversation.get_participant_contributions(p.get_id())
		p.set_eligible_contributions(contributions)

		Conversation.create_participant_conversation(conversation.lang, p)

		own_conversation = p.own_conversation

		print('Begin computing textual complexity')
		compute_indices(doc=own_conversation)
		print('Finished computing textual complexity')

		for key, value in own_conversation.indices.items():
			p.set_textual_index(repr(key), value)


def perform_sna(conversation: Conversation, needs_anonymization: bool):
	participants = conversation.get_participants()

	for i, p1 in enumerate(participants):
		for j, p2 in enumerate(participants):
			if i != j:
				current_value = p1.get_index(CNAIndices.OUTDEGREE)
				p1.set_index(CNAIndices.OUTDEGREE, current_value +
											conversation.get_score(p1.get_id(), p2.get_id()))

				current_value = p2.get_index(CNAIndices.INDEGREE)
				p2.set_index(CNAIndices.INDEGREE, current_value +
											conversation.get_score(p1.get_id(), p2.get_id()))
			else:
				current_value = p1.get_index(CNAIndices.OUTDEGREE)
				p1.set_index(CNAIndices.OUTDEGREE, current_value +
											conversation.get_score(p1.get_id(), p1.get_id()))



import csv
import numpy as np
import pandas as pd
from os import listdir
from os.path import isfile, join, splitext
from pathlib import Path

from sphinx.testing.path import path

from rb.summarization.utils.parser.custom_parser import CustomParser
from rb.summarization.systems import TextRank, Gensim, Submodular

parser = CustomParser().get_instance()

textrank_summarizer = TextRank()
gensim_summarizer = Gensim()
submodular_summarizer = Submodular()

path = Path("/home/vacioaca/Documents/summarization_input_danielle")

filenames = [f for f in listdir(path) if isfile(join(path, f))]

for filename in filenames:
    print(filename)
    df = pd.DataFrame(columns=['Text', 'Order', 'Sentence'])

    with open(path / filename, mode='r') as input_file:
        doc = input_file.read()
        sentences = parser.tokenize_sentences(doc)
        for index, sentence in enumerate(sentences):
            df = df.append({'Text': filename, 'Order': (index + 1), 'Sentence': sentence}, ignore_index=True)

    df.to_csv(path / 'csvs' / (splitext(filename)[0] + '.csv'), index=False)
    # break

# for filename in filenames:
#     print(filename)
#     df_path = path / 'csvs' / (splitext(filename)[0] + '.csv')
#     df = pd.read_csv(df_path)
#
#     with open(path / filename, mode='r') as input_file:
#         doc = input_file.read()

        # output_textrank = textrank_summarizer.summarize(doc, ratio=0.4)
        # df['textrank_0.4'] = pd.Series(output_textrank)
        # output_gensim = gensim_summarizer.summarize(doc, ratio=0.4)
        # df['gensim_0.4'] = pd.Series(output_gensim)
        # output_submodular = submodular_summarizer.summarize(doc, ratio=0.4)
        # df['submodular1_0.4'] = pd.Series(output_submodular)

    #     output_submodular = submodular_summarizer.summarize(doc, ratio=0.4)
    #     df['submodular2_0.4'] = pd.Series(output_submodular)
    #
    # df.to_csv(df_path, index=False)
    # break


# with open(path / 'summary.csv', mode='w') as csv_file:
#     fieldnames = ['Text', 'Order', 'Sentence']
#     writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
#     writer.writeheader()
#
#     for filename in filenames:
#         print(filename)
#         with open(path / filename, mode='r') as input_file:
#             doc = input_file.read()
            # output = textrank_summarizer.summarize(doc, ratio=0.25)
            # output = gensim_summarizer.summarize(doc, ratio=0.25)
            # output = submodular_summarizer.summarize(doc, ratio=0.25)
            # print(output)
            # print("\n")
        #     sentences = parser.tokenize_sentences(doc)
        #     for index, sentence in enumerate(sentences):
        #         writer.writerow({'Text': filename, 'Order': (index+1), 'Sentence': sentence})
        # break

"""
d = {
    "12_JobSearch.txt": 18,
    "15_PatientRights.txt": 17,
    "30_GlobalWarming.txt": 19,
    "20_Disabilities.txt": 20,
    "4_ChildSafety.txt": 16,
    "26_Hurricanes.txt": 18,
    "6_Smoking.txt": 12,
    "21_CheckCashing.txt": 18,
    "1_BodyScans.txt": 18,
    "8_OrganDonation.txt": 16,
    "29_Biosphere.txt": 16,
    "24_Hybrids.txt": 42,
    "11_CivilService.txt": 14,
    "2_ChildSeats.txt": 27,
    "3_HealthyBaby.txt": 13,
    "25_Google.txt": 18,
    "9_911.txt": 15,
    "7_SkinCancer.txt": 14,
    "28_CarPollution.txt": 18,
    "17_Crimes.txt": 13,
    "5_FireworksDanger.txt": 17,
    "14_ComputerVirus.txt": 17,
    "10_Internet.txt": 18,
    "23_InternetShopping.txt": 38
}


x = list(d.values())
print("\nOriginal array:")
print(x)
r1 = np.mean(x)
r2 = np.average(x)
assert np.allclose(r1, r2)
print("\nMean: ", r1)
r1 = np.std(x)
r2 = np.sqrt(np.mean((x - np.mean(x)) ** 2 ))
assert np.allclose(r1, r2)
print("\nstd: ", 1)
r1 = np.var(x)
r2 = np.mean((x - np.mean(x)) ** 2 )
assert np.allclose(r1, r2)
print("\nvariance: ", r1)
"""



from typing import List, Dict

from rb.core.lang import Lang
from rb.core.cscl.community import Community
from rb.processings.cscl.community_processing import CommunityProcessing
from rb.core.cscl.cscl_criteria import CsclCriteria

from rb.utils.rblogger import Logger

SECONDS_IN_DAY = 60 * 60 * 24
SECONDS_IN_MONTH = SECONDS_IN_DAY * 30

class CommunityTimeProcessing:

    def determine_subcommunities(self, community: Community, month_increment: int, day_increment: int):
        if (community.get_first_contribution_date() is None):
            return

        subcommunity_start = community.get_first_contribution_date()

        while True:
            # todo take into account length of month and current date
            subcommunity_end = subcommunity_start

            subcommunity_end += (month_increment * SECONDS_IN_MONTH)
            subcommunity_end += (day_increment * SECONDS_IN_DAY)

            if subcommunity_end < community.get_last_contribution_date():
                subcommunity = self.extract_subcommunity(parent_community=community,
                                                        subcommunity_start=subcommunity_start,
                                                        subcommunity_end=subcommunity_end)

                community.add_subcommunity(subcommunity)

                subcommunity_start = subcommunity_end
            else:
                break

        if subcommunity_start < community.get_last_contribution_date():
            subcommunity = self.extract_subcommunity(parent_community=community,
                                                    subcommunity_start=subcommunity_start,
                                                    subcommunity_end=community.get_last_contribution_date())

            community.add_subcommunity(subcommunity)
        

    def extract_subcommunity(self, parent_community: Community, subcommunity_start: int, subcommunity_end: int) -> Community:
        subcommunity = Community(lang=parent_community.lang,
                                community=parent_community.get_community_raw(),
                                start_date=subcommunity_start,
                                end_date=subcommunity_end)

        community_processing = CommunityProcessing()
        
        community_processing.determine_participant_contributions(subcommunity)
        community_processing.determine_participation(subcommunity)
        community_processing.compute_sna_metrics(subcommunity)

        return subcommunity

    def model_time_evoluton(self, community: Community):
        return


from pathlib import Path
from pyrouge import Rouge155

import logging
logging.disable(logging.INFO)


class Rouge(object):
    
    _INSTANCE = None
    
    def __init__(self, relative_path: Path = Path.cwd()):
        self._r = Rouge155()
        
        self._r.system_dir = str(relative_path / "system_summaries")     # your summaries
        self._r.model_dir = str(relative_path / "model_summaries")       # gold standard summaries
        
        self._r.system_filename_pattern = r'duc2002.(\d+).txt'
        self._r.model_filename_pattern = r'duc2002.[A-Z].#ID#.txt'
        
    @classmethod
    def get_instance(cls) -> "Rouge":
        """
        Note: Multiple calls of evaluate method with the same instance will throw "Illegal division by zero" exception.
        :return: unique instance of @Rouge class
        """
        if cls._INSTANCE is None:
            cls._INSTANCE = Rouge()
        return cls._INSTANCE
    
    @property
    def system_dir(self):
        return self._r.system_dir

    @property
    def model_dir(self):
        return self._r.model_dir
    
    def evaluate(self):
        """
        Evaluate and returns the result as a dictionary.
        :return: dictionary with results
        """
        options = "-e /home/vacioaca/ROUGE-1.5.5/data -c 95 -r 1000 -n 4 -w 1.2 -a -x"
        output = self._r.convert_and_evaluate(rouge_args=options)
        return output
        # return self._r.output_to_dict(output)


if __name__ == "__main__":
    rouge = Rouge()

    result = rouge.evaluate()
    print(result)



import itertools
import numpy as np
import wmd

from pathlib import Path
from rb.core.lang import Lang
from rb.parser.spacy_parser import SpacyParser
from rb.summarization.enums.feature_type import FeatureType
from rb.summarization.utils.parser.custom_parser import CustomParser
from sklearn.cluster import KMeans
from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer
from typing import List, Union


nlp = None


def tokenize(parser: SpacyParser, doc_sentences: List[str], lang: Lang):
    """
    Tokenize a list of strings using spaCy pipeline
    :param parser:
    :param doc_sentences:
    :param lang:
    :return: tuples of index and tokens computed for every string in the list
    """

    lower_sentences = map(lambda t: t.lower(), doc_sentences)
    for i, tokens in enumerate(parser.get_tokens_lemmas(lower_sentences, lang)):
        tokens = [token.lemma_.lower() for token in tokens if not token.is_stop and token.is_alpha and len(token.text) > 2]
        if len(set(tokens)) > 3:
            yield (i, tokens)


def tokenize2(parser: SpacyParser, doc_sentences: List[str], lang: Lang):
    """
    Tokenize a list of strings using spaCy model
    :param parser:
    :param doc_sentences:
    :param lang:
    :return:
    """
    global nlp
    if not nlp:
        nlp = parser.get_model(lang)
        nlp.add_pipe(wmd.WMD.SpacySimilarityHook(nlp), last=True)

    lower_sentences = map(lambda t: t.lower(), doc_sentences)
    for i, sentence in enumerate(lower_sentences):
        # tokens = [token for token in nlp(sentence) if not token.is_stop and token.is_alpha and len(token.text) > 2]
        # if len(tokens) > 3:
        #     yield (i, tokens)
        if len(set(sentence.split())) > 3:
            yield (i, nlp(sentence))


def build_feature_matrix(docs: List, feature_type: Union[FeatureType, str], ngram_range=(1, 1), min_df=0.0, max_df=1.0):
    """
    :param docs: list of texts
    :param feature_type: 'binary', 'frequency', 'tfidf'
    :param ngram_range:
    :param min_df:
    :param max_df:
    :return:
    """

    if isinstance(feature_type, str):
        feature_type = FeatureType(feature_type.lower().strip())

    if feature_type == FeatureType.BINARY:
        vectorizer = CountVectorizer(binary=True, ngram_range=ngram_range, min_df=min_df, max_df=max_df)
    elif feature_type == FeatureType.FREQUENCY:
        vectorizer = CountVectorizer(binary=False, ngram_range=ngram_range, min_df=min_df, max_df=max_df)
    elif feature_type == FeatureType.TFIDF:
        vectorizer = TfidfVectorizer(ngram_range=ngram_range, min_df=min_df, max_df=max_df, use_idf=True)
    else:
        raise Exception("Wrong feature type entered. Possible values: 'binary', 'frequency', 'tfidf'")

    feature_matrix = vectorizer.fit_transform(docs).astype(float)

    return vectorizer, feature_matrix


def k_means(feature_matrix, no_clusters: int = 5, max_iter: int = 300):

    km = KMeans(n_clusters=no_clusters, max_iter=max_iter).fit(feature_matrix)

    return km, km.labels_


def get_indices(lst, value):
    """
    Returns all positional indices of a value in a list
    :param lst: list of values
    :param value: value to be searched for
    :return: list of positional indices
    """

    return [i for (el, i) in zip(lst, range(len(lst))) if value == el]


def list_intersection(lst1, lst2):
    """
    :param lst1: list of values
    :param lst2:  list of values
    :return: common unique values
    """

    return list(set(lst1) & set(lst2))


def write_list_to_file(file_path: Path, var_list: List[str]):
    with open(str(file_path), 'w', encoding='utf-8') as f:
        for el in var_list:
            f.write(el + '\n')


def write_str_to_file(file_path: Path, var_str: str):
    with open(str(file_path), 'w', encoding='utf-8') as f:
        f.write(var_str)


def append_list_to_file(file_path: Path, var_list: List[str]):
    with open(str(file_path), 'a', encoding='utf-8') as f:
        for el in var_list:
            f.write(el + '\n')


"""
            Similarity metrics below 
"""

DELTA = 0.85
ETA = 0.4


def most_similar_token(joint_token, tokens):
    max_sim = -1.0
    sim_token = None

    if not joint_token or not joint_token.vector_norm:
        return sim_token, max_sim

    for ref_token in tokens:
        # check if token is a valid spaCy object and has a word vector
        if ref_token and ref_token.vector_norm:
            sim = joint_token.similarity(ref_token)
            if sim > max_sim:
                max_sim = sim
                sim_token = ref_token
    return sim_token, max_sim


def word_order_vector(tokens, joint_tokens):
    wovec = np.zeros(len(joint_tokens))
    for i, joint_token in enumerate(joint_tokens):
        if joint_token in tokens:
            wovec[i] = tokens.index(joint_token)
        else:
            sim_token, max_sim = most_similar_token(joint_token, tokens)
            if max_sim > ETA:
                wovec[i] = tokens.index(sim_token)
            else:
                wovec[i] = 0
    return wovec


def word_order_similarity(doc1, doc2):
    """

    :param doc1: list of spaCy tokens
    :param doc2: list of spaCy tokens
    :return: word order similarity score
    """

    doc1_dict = {token.text: token for token in doc1}
    doc2_dict = {token.text: token for token in doc2}
    joint_tokens = {**doc1_dict, **doc2_dict}.values()  # dictionary merge syntax valid in python >= 3.5
    r1 = word_order_vector(doc1, joint_tokens)
    r2 = word_order_vector(doc2, joint_tokens)

    # check if both arrays have only zeros
    if np.count_nonzero(r1) == 0 and np.count_nonzero(r2) == 0:
        return 0.0

    return 1.0 - (np.linalg.norm(r1 - r2) / np.linalg.norm(r1 + r2))


def word_embeddings_similarity(doc1, doc2):
    vector1 = np.zeros(300)
    vector2 = np.zeros(300)
    for token in doc1:
        # if not token.is_stop and token.is_alpha:
        vector1 = vector1 + token.vector
    vector1 = np.divide(vector1, len(doc1))
    for token in doc2:
        # if not token.is_stop and token.is_alpha:
        vector2 = vector2 + token.vector
    vector2 = np.divide(vector2, len(doc2))

    if np.count_nonzero(vector1) == 0 or np.count_nonzero(vector2) == 0:
        return 0.0

    return np.dot(vector1, vector2) / (np.linalg.norm(vector1 * np.linalg.norm(vector2)))


def combined_similarity(docs):
    """
    Linear combination similarity between word embeddings similarity and word order similarity.
    :param docs:
    :return:
    """

    n = len(docs)
    similarity_matrix = np.identity(n, float)
    for i, j in itertools.combinations(range(n), 2):
        sim = DELTA * word_embeddings_similarity(docs[i], docs[j]) + \
              (1.0 - DELTA) * word_order_similarity(docs[i], docs[j])
        similarity_matrix[i, j] = similarity_matrix[j, i] = sim
    return similarity_matrix


def sentence_similarity(sentence_1: str, sentence_2: str, parser: SpacyParser, lang: Lang = Lang.EN):
    nlp = parser.get_model(lang)
    doc1 = nlp(sentence_1)
    doc2 = nlp(sentence_2)
    return DELTA * word_embeddings_similarity(doc1, doc2) + (1.0 - DELTA) * word_order_similarity(doc1, doc2)


def generate_gensim_system_summaries(spacy_parser, dataset_parser):

    import warnings

    with warnings.catch_warnings():
        warnings.filterwarnings(action="ignore", category=UserWarning, module="gensim")
        warnings.filterwarnings(action="ignore", category=DeprecationWarning)
        from gensim.summarization import summarize

    i = 0
    for _, docs in dataset_parser.get_docs_with_summary().items():
        for doc in docs:
            print("Processing %s in iteration %d" % (doc, i))
            doc_text = dataset_parser.get_document_text(doc)
            model_summary_text = dataset_parser.get_model_summary(doc)

            model_summary_as_list = spacy_parser.tokenize_sentences(model_summary_text)
            model_summary_as_str = '\n'.join(model_summary_as_list)

            gensim_summary_as_str = '\n'.join(summarize(doc_text, ratio=0.2, split=True))

            model_summary_filename = "duc2001.A." + "{0:0=3d}".format(i + 1) + ".txt"
            system_summary_filename = "duc2001." + "{0:0=3d}".format(i + 1) + ".txt"

            rouge_path = Path.cwd() / ".." / "rouge"
            write_str_to_file(rouge_path / "model_summaries" / model_summary_filename, model_summary_as_str)
            write_str_to_file(rouge_path / "system_summaries" / system_summary_filename, gensim_summary_as_str)

            i = i + 1


def main():
    parser = CustomParser().get_instance()
    sentence1 = "A quick brown dog jumps over the lazy fox."
    sentence2 = "A quick brown fox jumps over the lazy dog."

    # tokenized_sentences = list(tokenize2(parser, [sentence1, sentence2], Lang.EN))

    # print(word_order_similarity(tokenized_sentences[0][1], tokenized_sentences[1][1]))

    # sentence1 = "Obama speaks to the media in Illinois."
    # sentence2 = "The president greets the press in Chicago."
    # res = sentence_similarity(sentence1, sentence2, parser, Lang.EN)
    # print(res)

    # dataset_parser = DUC2001Parser(Path.cwd() / ".." / "corpus"/ "DUC2001")
    # generate_gensim_system_summaries(parser, dataset_parser)


if __name__ == "__main__":

    main()



from rb.summarization.systems.summarizer_type import SummarizerType
from rb.summarization.systems.text_rank import TextRank
from rb.summarization.systems.submodular import Submodular
from rb.summarization.systems.gensim_local import Gensim
